//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017-2019 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package auth

import (
	"crypto/rand"
	"encoding/base64"
	"sync"
	"time"
)

const (
	defaultAge = 24 * time.Hour
)

type Session struct {
	username string
	groups   []string
}

func generateSessionId() (string, error) {
	var b [32]byte
	if _, err := rand.Read(b[:]); err != nil {
		return "", err
	}
	return base64.RawURLEncoding.EncodeToString(b[:]), nil
}

type SessionManager struct {
	maxAge   time.Duration
	mutex    sync.RWMutex
	sessions map[string]*Session
}

func NewSessionManager(cfg SessionsConfig) (sm *SessionManager, err error) {
	sm = &SessionManager{maxAge: defaultAge}
	if cfg.MaxAge > 0 {
		sm.maxAge = cfg.MaxAge
	}
	sm.sessions = make(map[string]*Session)
	return
}

func (sm *SessionManager) NewSession() (id string, err error) {
	if id, err = generateSessionId(); err != nil {
		return
	}

	sm.mutex.Lock()
	defer sm.mutex.Unlock()
	sm.sessions[id] = &Session{}
	return
}
